package observer;

import java.util.Observable;
import java.util.Observer;

public aspect ObservingOpen {
	private interface BreakfastTaker {
		public void breakfastTime();
	}
	declare parents: (Bee || Hummingbird) implements BreakfastTaker;

	static class OpenNotifier extends Observable {
		private Flower _enclosing;
		private boolean alreadyOpen = false;
		public OpenNotifier(Flower flower) {
			_enclosing = flower;
		}
		public void notifyObservers() {
			if(_enclosing.isOpen() && !this.alreadyOpen) {
				this.setChanged();
				super.notifyObservers();
				this.alreadyOpen = true;
			}
		}
		public void close() {
			this.alreadyOpen = false;
		}
	}
	static class OpenObserver implements Observer {
		private BreakfastTaker _enclosing;
		public OpenObserver(BreakfastTaker enclosing) {
			_enclosing = enclosing;
		}
		public void update(Observable ob, Object a) {
			_enclosing.breakfastTime();
		}
	}
	private	OpenNotifier Flower.oNotify = new OpenNotifier(this);
	private OpenObserver Hummingbird.openObsrv = new OpenObserver(this);
	private OpenObserver Bee.openObsrv = new OpenObserver(this);

	public Observable Flower.opening() {
		return oNotify;
	}
	pointcut flowerOpen(Flower flower):
		execution(void open()) && this(flower);
	after(Flower flower) returning : flowerOpen(flower) {
		flower.oNotify.notifyObservers();
	}
	pointcut flowerClose(Flower flower):
		execution(void close()) && this(flower);
	after(Flower flower): flowerClose(flower) {
		flower.oNotify.close();
	}

	public Observer Bee.openObserver() { 
		return openObsrv; 
	}
	public java.util.Observer Hummingbird.openObserver() { 
		return openObsrv;
	}
}
