package observer;

import java.util.WeakHashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.ListIterator;

public aspect ObservingOpen {
	private interface Subject {}
	private interface Observer {}

	declare parents: Flower implements Subject;
	declare parents: (Bee || Hummingbird) implements Observer;

	public abstract boolean Subject.isOpen();
	public abstract void Observer.breakfastTime();
	private boolean Subject.alreadyOpen = false;

	private WeakHashMap subject2ObserversMap = new WeakHashMap();
	private List getObservers(Subject subject) {
		List observers = (List)subject2ObserversMap.get(subject);
		if(observers == null) {
			observers = new ArrayList();
			subject2ObserversMap.put(subject, observers);
		}
		return observers;
	}
	public void addObserver(Subject subject, Observer observer) {
		List observers = getObservers(subject);
		if(!observers.contains(observer))
			observers.add(observer);
		subject2ObserversMap.put(subject, observers);
	}
	public void removeObserver(Subject subject, Observer observer) {
		getObservers(subject).remove(observer);
	}
	public void clearObservers(Subject subject) {
		getObservers(subject).clear();
	}
	private void notifyObservers(Subject subject) {
		if(subject.isOpen() && !subject.alreadyOpen) {
			subject.alreadyOpen = true;
			List observers = getObservers(subject);
			for(ListIterator it = observers.listIterator(); it.hasNext();) {
				((Observer)it.next()).breakfastTime();
			}
		}
	}
	pointcut flowerOpen(Subject subject):
		execution(void open()) && this(subject);
	after(Subject subject) returning : flowerOpen(subject) {
		//subject.oNotify.notifyObservers();
		notifyObservers(subject);
	}
	pointcut flowerClose(Subject subject):
		execution(void close()) && this(subject);
	after(Subject subject): flowerClose(subject) {
		//subject.oNotify.close();
		subject.alreadyOpen = false;
	}

//	declare warning: call(java.util.Observable opening()):
//		"opening() called here.";

//	static class OpenNotifier extends java.util.Observable {
//		private Subject _enclosing;
//		private boolean alreadyOpen = false;
//		public OpenNotifier(Subject flower) {
//			_enclosing = flower;
//		}
//		public void notifyObservers() {
//			if(_enclosing.isOpen() && !this.alreadyOpen) {
//				this.setChanged();
//				super.notifyObservers();
//				this.alreadyOpen = true;
//			}
//		}
//		public void close() {
//			this.alreadyOpen = false;
//		}
//	}
//	static class OpenObserver implements java.util.Observer {
//		private Observer _enclosing;
//		public OpenObserver(Observer enclosing) {
//			_enclosing = enclosing;
//		}
//		public void update(java.util.Observable ob, Object a) {
//			_enclosing.breakfastTime();
//		}
//	}
//	private	OpenNotifier Subject.oNotify = new OpenNotifier(this);
//	private OpenObserver Observer.openObsrv = new OpenObserver(this);
//	public java.util.Observable Subject.opening() {
//		return oNotify;
//	}
//	public java.util.Observer Observer.openObserver() { 
//		return openObsrv;
//	}
}