package observer;

import java.util.WeakHashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.ListIterator;

public aspect ObservingClose {
	private interface Subject {}
	private interface Observer {}

	public abstract boolean Subject.isOpen();
	public abstract void Observer.bedtimeSleep();
	private boolean Subject.alreadyClosed = false;

	/************************************************************/
	private WeakHashMap subject2ObserversMap = new WeakHashMap();
	private List getObservers(Subject subject) {
		List observers = (List)subject2ObserversMap.get(subject);
		if(observers == null) {
			observers = new ArrayList();
			subject2ObserversMap.put(subject, observers);
		}
		return observers;
	}
	public void addObserver(Subject subject, Observer observer) {
		List observers = getObservers(subject);
		if(!observers.contains(observer))
			observers.add(observer);
		subject2ObserversMap.put(subject, observers);
	}
	public void removeObserver(Subject subject, Observer observer) {
		getObservers(subject).remove(observer);
	}
	public void clearObservers(Subject subject) {
		getObservers(subject).clear();
	}
	private void notifyObservers(Subject subject) {
		if(!subject.isOpen() && !subject.alreadyClosed) {
			subject.alreadyClosed = true;
			List observers = getObservers(subject);
			for(ListIterator it = observers.listIterator(); it.hasNext();) {
				((Observer)it.next()).bedtimeSleep();
			}
		}
	}
	/************************************************************/
	pointcut flowerOpen(Subject subject):
		execution(void open()) && this(subject);
	after(Subject subject) returning : flowerOpen(subject) {
		subject.cNotify.open();
		subject.alreadyClosed = false;
	}
	pointcut flowerClose(Subject subject):
		execution(void close()) && this(subject);
	after(Subject subject): flowerClose(subject) {
		subject.cNotify.notifyObservers();
		notifyObservers(subject);
	}
	/************************************************************/
	declare warning: call(java.util.Observable closing()):
		"closing() called here.";
	/************************************************************/
	static class CloseNotifier extends java.util.Observable {
		private Subject _enclosing;
		private boolean alreadyClosed = false;
		public CloseNotifier(Subject flower) {
			_enclosing = flower;
		}
		public void notifyObservers() {
			if(!_enclosing.isOpen() && !this.alreadyClosed) {
				this.setChanged();
				super.notifyObservers();
				this.alreadyClosed = true;
			}
		}
		public void open() {
			this.alreadyClosed = false;
		}
	}
	static class CloseObserver implements java.util.Observer {
		private Observer _enclosing;
		public CloseObserver(Observer enclosing) {
			_enclosing = enclosing;
		}
		public void update(java.util.Observable ob, Object a) {
			_enclosing.bedtimeSleep();
		}
	}
	private CloseNotifier Subject.cNotify = new CloseNotifier(this);
	private CloseObserver Observer.closeObsrv = new CloseObserver(this);
	public java.util.Observable Subject.closing() {
		return cNotify;
	}
	public java.util.Observer Observer.closeObserver() { 
		return closeObsrv;
	}

	declare parents: Flower implements Subject;
	declare parents: (Bee || Hummingbird) implements Observer;
}