package observer;

import java.util.WeakHashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.ListIterator;
/**
 * @author Miguel Pessoa Monteiro
 */
public abstract aspect ObservingRelationships {
	protected interface Subject {}
	protected interface Observer {}

	protected WeakHashMap subject2ObserversMap = new WeakHashMap();
	protected List getObservers(Subject subject) {
		List observers = (List)subject2ObserversMap.get(subject);
		if(observers == null) {
			observers = new ArrayList();
			subject2ObserversMap.put(subject, observers);
		}
		return observers;
	}
	public void addObserver(Subject subject, Observer observer) {
		List observers = getObservers(subject);
		if(!observers.contains(observer))
			observers.add(observer);
		subject2ObserversMap.put(subject, observers);
	}
	public void removeObserver(Subject subject, Observer observer) {
		getObservers(subject).remove(observer);
	}
	public void clearObservers(Subject subject) {
		getObservers(subject).clear();
	}
	protected abstract void notifyObservers(Subject subject);
}